/*
 * ioGame
 * Copyright (C) 2021 - 2023  渔民小镇 （262610965@qq.com、luoyizhu@gmail.com） . All Rights Reserved.
 * # iohao.com . 渔民小镇
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.iohao.mmo.equip.entity;

import com.iohao.mmo.common.kit.RandomKit;
import lombok.AccessLevel;
import lombok.Builder;
import lombok.Data;
import lombok.experimental.FieldDefaults;
import org.springframework.data.mongodb.core.mapping.Document;
import java.math.BigDecimal;

/**
 * 装备固定属性
 * <pre>
 *     装备的基本属性
 * </pre>
 *
 * @author 唐斌
 * @date 2023-07-24
 */
@Data
@Builder
@Document
@FieldDefaults(level = AccessLevel.PRIVATE)
public class FixedEquipProperty {

    /** 物理攻击最小值 */
    int physicsAttackMin;
    /** 物理攻击最大值 */
    int physicsAttackMax;
    /** 法术攻击最小值 */
    int magicAttackMin;
    /** 法术攻击最大值 */
    int magicAttackMax;

    /** 物理防御 */
    int physicsDefense;
    /** 法术防御 */
    int magicDefense;

    /** 物理命中 */
    int physicsHit;
    /** 物理闪避 */
    int physicsDodge;
    /** 法术命中 */
    int magicHit;
    /** 法术闪避 */
    int magicDodge;

    /** 生命值百分比 */
    BigDecimal hpPct;

    /** 耐久度 */
    int durability;

    public static FixedEquipProperty randomFixed(FixedEquipProperty fixedEquipPropertyMin,
                                                 FixedEquipProperty fixedEquipPropertyMax){
        if(fixedEquipPropertyMin==null || fixedEquipPropertyMax == null)
            return FixedEquipProperty.builder().build();
        return FixedEquipProperty.builder()
                .physicsAttackMin(
                        RandomKit.randomFromInt(fixedEquipPropertyMin.getPhysicsAttackMin(),
                                fixedEquipPropertyMax.getPhysicsAttackMin()))
                .physicsAttackMax(
                        RandomKit.randomFromInt(fixedEquipPropertyMin.getPhysicsAttackMax(),
                                fixedEquipPropertyMax.getPhysicsAttackMax()))
                .magicAttackMin(
                        RandomKit.randomFromInt(fixedEquipPropertyMin.getMagicAttackMin(),
                                fixedEquipPropertyMax.getMagicAttackMin()))
                .magicAttackMax(
                        RandomKit.randomFromInt(fixedEquipPropertyMin.getMagicAttackMax(),
                                fixedEquipPropertyMax.getMagicAttackMax()))
                .physicsDefense(
                        RandomKit.randomFromInt(fixedEquipPropertyMin.getPhysicsDefense(),
                                fixedEquipPropertyMax.getPhysicsDefense()))
                .magicDefense(
                        RandomKit.randomFromInt(fixedEquipPropertyMin.getMagicDefense(),
                                fixedEquipPropertyMax.getMagicDefense()))
                .magicDefense(
                        RandomKit.randomFromInt(fixedEquipPropertyMin.getMagicDefense(),
                                fixedEquipPropertyMax.getMagicDefense()))
                .physicsHit(
                        RandomKit.randomFromInt(fixedEquipPropertyMin.getPhysicsHit(),
                                fixedEquipPropertyMax.getPhysicsHit()))
                .physicsDodge(
                        RandomKit.randomFromInt(fixedEquipPropertyMin.getPhysicsDodge(),
                                fixedEquipPropertyMax.getPhysicsDodge()))
                .magicHit(
                        RandomKit.randomFromInt(fixedEquipPropertyMin.getMagicHit(),
                                fixedEquipPropertyMax.getMagicHit()))
                .magicDodge(
                        RandomKit.randomFromInt(fixedEquipPropertyMin.getMagicDodge(),
                                fixedEquipPropertyMax.getMagicDodge()))
                .hpPct(
                        RandomKit.randomFromBigDecimal(fixedEquipPropertyMin.getHpPct(),
                                fixedEquipPropertyMax.getHpPct()))
                .durability(
                        RandomKit.randomFromInt(fixedEquipPropertyMin.getDurability(),
                                fixedEquipPropertyMax.getDurability()))
                .build();
    }
}
